function [J,lambda,u,X0,P,D,e]=reduction_quadrique(A,b,c,nsy);


% Rduction de quadriques dans R^n
%
%  Cette fonction permet de rduire des quadriques dans R^n d'quations : 
%   tXAX+tBX+c=0 o X=(x1,...,xn), A est une matrice relle symtrique, 
%                                  b un vecteur colonne
%                                  et c un scalaire.
%  Des tests de vrification accompagnent le calcul.
%
%
%  [J,lambda,u,X0,P,D,e]=reduction_quadrique(A,b,c,nsy) :  
%
% * variables d'entres : 
%     * La matrice carre A
%     * la matrice colonne b
%     * le rel c
%     * un entier nsy dans {0,1} : nsy=1 calcul en symbolique
%                                  nsy=0 en numrique.
% * variables de sortie : 
%     a) premier cas A est inversible ( centre) 
%        * X0 : le vecteurs des coordonnes du centre de la quadrique.
%        * P : la matrice orthogonale de changement de base vectorielle : 
%        * D : la matrice des valeurs propres de A
%        * e : un scalaire
%         tels que dans le nouveau repre (X0,P), l'quation rduite de la quadrique soit : 
%               tZDZ+e=0
%        * J,lambda,u sont les matrices vides.
%     b) second cas : A n'est pas inversible
%        * X0 : le vecteurs des coordonnes du centre (partiel) de la quadrique.
%        * P : la matrice orthogonale de changement de base vectorielle : 
%        * D : la matrice des valeurs propres de A
%        * e : un scalaire
%        * J : un tableau d'indices de {1,...,n}
%        * lambda : un tableau de rel indic par J
%        * u : un tableau de rel indic par {1,...,n}\J
%      tels que dans le nouveau repre (X0,P), l'quation rduite de la quadrique soit : 
%               sum_{i\in J} lambda_i z_i^2+sum_{i\in {1,...,n}\J} u_i z_i+e=0
%
%
%
% ************ Fonctions auxiliaires utilises ************
%
%     aucune
%
% *********************************************************
%
% (c) 2005 by      Jrme BASTIEN,
%                  UTBM, Dpartement GMC, Belfort
%                  E-Mail : jerome.bastien@utbm.fr


if max(max(abs(A)))==0
   disp('Attention'); 
   error('A est nulle : arrt du programme');
end
n=length(A);
[P,R]=eig(A);
if nsy==1
   bs=sym(b);
   cs=sym(c);
   P=sym(P);
   R=sym(R);
   As=sym(A);
else
   bs=b;
   cs=c;
   As=A;
end   
D=diag(R);
alpha=sqrt(sum((P(:,1)).^2));
P=P/alpha;
disp('Vrification');
disp('Orthogonalit');
disp(P'*P);
disp('Changement de base');
disp(P'*As*P);
disp(D);
disp(' ');
disp('Appuyez sur une touche pour continuer');
pause;
% changement d'origine
ddeta=det(As);
if (ddeta~=0)
    X0=-(As\bs)/2;
    e=(b'*X0)/2+cs;
    J=[];
    lambda=[];
    u=[];
else
    u=P'*b;
    I=find(D==0);
    J=find(D~=0);
    if nsy==1
       X0=sym(zeros(n,1));
    else
       X0=zeros(n,1);
    end
    X0(J)=-(((1./D(J)))/2).*u(J);  
    e=X0'*(D.*X0)+b'*P*X0+c;
    lambda=D(J);
    u=u(I);   
end
